///////////////////////////////////////////////////////////////////////
//                                                                   //
//   ImagePanel.cs                                                   //
//                                                                   //
//   C# source code for user control displaying simple graphic       //
//                                                                   //
//   Copyright (C) 2007, by Matt Roberts, KK5JY,                     //
//   All rights reserved.                                            //
//                                                                   //
///////////////////////////////////////////////////////////////////////


// system modules
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace SimpleImageViewer {
	//
	//
	//   class ImagePanel
	//
	//
	public partial class ImagePanel : UserControl, IDisposable {
		#region Internal Fields
		private Image CurrentImage;
		private Bitmap imageArea;
		private Brush background;
		private Point Origin;

		// flag to indicate desire to preserve aspect ratio
		private bool preserveRatio;
		#endregion

		#region Public Instance Constructors
		public ImagePanel() {
			InitializeComponent();
			CurrentImage = null;
			background = new SolidBrush(SystemColors.AppWorkspace);
			Origin = new Point(0, 0);
			preserveRatio = false;
		}

		public new void Dispose() {
			if (imageArea != null) {
				imageArea.Dispose();
			}
			if (background != null) {
				background.Dispose();
			}

			// chain to base
			base.Dispose();
		}

		#endregion

		#region Public Properties

		// get/set image to display
		public Image Image {
			get { return CurrentImage; }
			set {
				CurrentImage = value;
				if (Visible) {
					Invalidate();
				}
			}
		}

		// get/set "preserve aspect ratio" flag
		public bool PreserveAspectRatio {
			get { return preserveRatio; }
			set {
				preserveRatio = value;
				if (CurrentImage != null) {
					Invalidate();
				}
			}
		}

		#endregion

		#region Event Handlers

		// OnPaint(...) handler
		protected override void OnPaint(PaintEventArgs e) {
			if (imageArea == null || imageArea.Size != ClientRectangle.Size) {
				imageArea = new Bitmap(
					ClientRectangle.Width,
					ClientRectangle.Height);
			}

			// get a Graphics object pointed at the imageArea object
			Graphics graphics = Graphics.FromImage(imageArea);

			// fill it with the background color
			graphics.FillRectangle(background, ClientRectangle);

			// if there is an image to display, draw that on top
			if (CurrentImage != null) {
				Point where = Origin;
				Image toDraw = this.CurrentImage;
				bool disposeToDraw = false;

				// reduce the image, if it is to big for the viewport
				if (CurrentImage.Width > ClientRectangle.Width ||
					CurrentImage.Height > ClientRectangle.Height) {
					if (preserveRatio) {
						int overX = CurrentImage.Width - ClientRectangle.Width;
						int overY = CurrentImage.Height - ClientRectangle.Height;
						float aspect = (float)CurrentImage.Width / (float)CurrentImage.Height;
						Size newSize;
						if (overX > overY) {
							// resize so that X fits, Y will then center
							newSize = new Size(
								ClientRectangle.Width,
								(int)((float)ClientRectangle.Width * aspect));
						} else {
							// resize so that Y fits, X will then center
							newSize = new Size(
								(int)((float)ClientRectangle.Height / aspect),
								ClientRectangle.Height);
						}

						toDraw = new Bitmap(CurrentImage, newSize);
					} else {
						toDraw = new Bitmap(CurrentImage, ClientRectangle.Size);
					}
					disposeToDraw = true;
				}

				// center up, if the image is smaller than the viewport
				if (toDraw.Width < ClientRectangle.Width)
					where.X = (ClientRectangle.Width - toDraw.Width) / 2;
				if (toDraw.Height < ClientRectangle.Height)
					where.Y = (ClientRectangle.Height - toDraw.Height) / 2;

				// draw the real image into the imageArea
				graphics.DrawImage(toDraw, where);

				// dispose the temp image, if needed
				if (disposeToDraw) {
					toDraw.Dispose();
				}
			}

			// draw the imageArea into the control
			e.Graphics.DrawImage(imageArea, Origin);

			// chain to base
			base.OnPaint(e);
		}

		protected override void OnSizeChanged(EventArgs e) {
			// just invalidate the control
			Invalidate();

			// chain to base
			base.OnSizeChanged(e);
		}

		protected override void OnPaintBackground(PaintEventArgs e) {
			// don't paint background
			//base.OnPaintBackground(e);
		}

		protected override bool IsInputKey(Keys keyData) {
			return true;
		}

		#endregion
	}
}
